@@ -18,7 +18,7 @@ use std::error;
 use std::fmt;
 use std::cmp::min;
 use std::iter;
-use smallvec::SmallVec;
+use tinyvec::TinyVec;
 use ::ilog;
 use ::bitpacking::BitpackCursor;
 use ::header::{Codebook, Floor, FloorTypeZero, FloorTypeOne,
@@ -51,24 +51,17 @@ impl From<()> for AudioReadError {
 	}
 }
 
-impl error::Error for AudioReadError {
-	fn description(&self) -> &str {
-		match self {
-			&AudioReadError::EndOfPacket => "End of packet reached.",
-			&AudioReadError::AudioBadFormat => "Invalid audio packet",
-			&AudioReadError::AudioIsHeader => "The vorbis version is not supported",
-			&AudioReadError::BufferNotAddressable => "Requested to create buffer of non-addressable size",
-		}
-	}
-
-	fn cause(&self) -> Option<&error::Error> {
-		None
-	}
-}
+impl error::Error for AudioReadError {}
 
 impl fmt::Display for AudioReadError {
 	fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-		write!(fmt, "{}", error::Error::description(self))
+		let description = match self {
+			AudioReadError::EndOfPacket => "End of packet reached.",
+			AudioReadError::AudioBadFormat => "Invalid audio packet",
+			AudioReadError::AudioIsHeader => "The vorbis version is not supported",
+			AudioReadError::BufferNotAddressable => "Requested to create buffer of non-addressable size",
+		};
+		write!(fmt, "{}", description)
 	}
 }
 
@@ -120,48 +113,48 @@ fn floor_zero_decode(rdr :&mut BitpackCursor, codebooks :&[Codebook],
 	// TODO this needs to become 128 bits wide, not just 64,
 	// as floor0_amplitude_bits can be up to 127.
 	let amplitude = try!(rdr.read_dyn_u64(fl.floor0_amplitude_bits));
-	if amplitude > 0 {
-		let booknumber = try!(rdr.read_dyn_u32(
-			::ilog(fl.floor0_number_of_books as u64)));
-		match fl.floor0_book_list.get(booknumber as usize) {
-			// Undecodable per spec
-			None => try!(Err(FloorSpecialCase::PacketUndecodable)),
-			Some(codebook_idx) => {
-				let mut coefficients = Vec::with_capacity(fl.floor0_order as usize);
-				let mut last = 0.0;
-				let codebook = &codebooks[*codebook_idx as usize];
-				loop {
-					let mut last_new = last;
-					let temp_vector = try!(rdr.read_huffman_vq(codebook));
-					if temp_vector.len() + coefficients.len() < fl.floor0_order as usize {
-						// Little optimisation: we don't have to care about the >= case here
-						for &e in temp_vector {
-							coefficients.push((last + e as f32).cos());
-							last_new = e as f32;
-						}
-					} else {
-						for &e in temp_vector {
-							coefficients.push((last + e as f32).cos());
-							last_new = e as f32;
-							// This rule makes sure that coefficients doesn't get
-							// larger than floor0_order and saves an allocation
-							// in this case
-							if coefficients.len() == fl.floor0_order as usize {
-								return Ok((coefficients, amplitude));
-							}
-						}
-					}
-					last += last_new;
-					if coefficients.len() >= fl.floor0_order as usize {
-						return Ok((coefficients, amplitude));
-					}
-				}
-			},
-		}
-	} else {
+	if amplitude <= 0 {
 		// This channel is unused in this frame,
 		// its all zeros.
-		try!(Err(FloorSpecialCase::Unused));
+		return Err(FloorSpecialCase::Unused);
+	}
+
+	let booknumber = try!(rdr.read_dyn_u32(
+		::ilog(fl.floor0_number_of_books as u64)));
+	match fl.floor0_book_list.get(booknumber as usize) {
+		// Undecodable per spec
+		None => try!(Err(FloorSpecialCase::PacketUndecodable)),
+		Some(codebook_idx) => {
+			let mut coefficients = Vec::with_capacity(fl.floor0_order as usize);
+			let mut last = 0.0;
+			let codebook = &codebooks[*codebook_idx as usize];
+			loop {
+				let mut last_new = last;
+				let temp_vector = try!(rdr.read_huffman_vq(codebook));
+				if temp_vector.len() + coefficients.len() < fl.floor0_order as usize {
+					// Little optimisation: we don't have to care about the >= case here
+					for &e in temp_vector {
+						coefficients.push((last + e as f32).cos());
+						last_new = e as f32;
+					}
+				} else {
+					for &e in temp_vector {
+						coefficients.push((last + e as f32).cos());
+						last_new = e as f32;
+						// This rule makes sure that coefficients doesn't get
+						// larger than floor0_order and saves an allocation
+						// in this case
+						if coefficients.len() == fl.floor0_order as usize {
+							return Ok((coefficients, amplitude));
+						}
+					}
+				}
+				last += last_new;
+				if coefficients.len() >= fl.floor0_order as usize {
+					return Ok((coefficients, amplitude));
+				}
+			}
+		},
 	}
 	unreachable!();
 }
@@ -259,41 +252,45 @@ fn floor_one_decode(rdr :&mut BitpackCursor, codebooks :&[Codebook],
 	return Ok(floor1_y);
 }
 
-fn extr_neighbor<F>(v :&[u32], x :usize,
-		compare :F, s :&'static str) -> (usize, u32)
-		where F :Fn(u32, u32) -> bool {
-	let bound = v[x];
-	let sm = v.split_at(x as usize).0;
+fn extr_neighbor<F>(v :&[u32], max_idx :usize,
+		compare :F, relation :&str) -> (usize, u32)
+		where F :Fn(u32, u32) -> std::cmp::Ordering {
+	use std::cmp::Ordering;
+
+	let bound = v[max_idx];
+	let prefix = &v[..max_idx];
+	let smaller = |a, b| compare(a, b) == Ordering::Less;
+
 	// First find a first index that fulfills
-	// the criterion of being "smaller" than bound;
-	// If "a smaller b" means compare(a, b) == true
-	let (mut extr_idx, mut max_val) = (|| {
-		for tu in sm.iter().cloned().enumerate() {
-			if compare(tu.1, bound) { return tu; }
-		}
-		panic!("No index y < {} found where v[y] is {} than v[{}] = 0x{:08x}!",
-			x, s, x, bound);
-	}) ();
-	// Now search for "bigger" entries;
-	// If "a bigger b" means compare(b, a) == true
-	let split_idx = extr_idx;
-	let smm = sm.split_at(split_idx).1;
-	for (idx, val) in smm.iter().cloned().enumerate() {
-		if compare(val, bound) && compare(max_val, val) {
-			extr_idx = idx + split_idx;
-			max_val = val;
-		}
-	}
-	return (extr_idx, max_val);
+	// the criterion of being "smaller" than bound
+	let min_idx = prefix.iter()
+		.position(|&val| smaller(val, bound))
+		.unwrap_or_else(||
+			panic!("No index y < {0} found where v[y] is {1} than v[{0}] = 0x{2:08x}!",
+				max_idx, relation, bound));
+
+	// Now search for "bigger" entries
+	let (offset, max_neighbor) = prefix[min_idx..].iter().cloned()
+		.enumerate()
+		// According to documentation of Iterator::max_by,
+		// "If several elements are equally maximum, the last element is returned".
+		// Thus, in order to find the *first* maximum element,
+		// we need to search from the end of `prefix`
+		.rev()
+		.filter(|&(_i, val)| smaller(val, bound))
+		.max_by(|&(_, a), &(_, b)| compare(a, b))
+		.unwrap_or((0, v[min_idx]));
+
+	(min_idx + offset, max_neighbor)
 }
 
 fn low_neighbor(v :&[u32], x :usize) -> (usize, u32) {
-	return extr_neighbor(v, x, |a, b| {a < b}, "smaller");
+	extr_neighbor(v, x, |a, b| a.cmp(&b), "smaller")
 }
 
 
 fn high_neighbor(v :&[u32], x :usize) -> (usize, u32) {
-	return extr_neighbor(v, x, |a, b| {a > b}, "bigger");
+	extr_neighbor(v, x, |a, b| b.cmp(&a), "bigger")
 }
 
 #[test]
@@ -419,13 +416,13 @@ fn floor_one_curve_compute_amplitude(floor1_y :&[u32], fl :&FloorTypeOne) -> (Ve
 			floor1_step2_flag.push(true);
 			floor1_final_y.push(if val >= room {
 				if highroom > lowroom {
-					(predicted + val - lowroom)
+					predicted + val - lowroom
 				} else {
-					(predicted - val + highroom - 1)
+					predicted - val + highroom - 1
 				}
 			} else {
-				(predicted + (if val % 2 == 1 {
-					- val - 1 } else { val } >> 1))
+				predicted + (if val % 2 == 1 {
+					- val - 1 } else { val } >> 1)
 			} as u32);
 		} else {
 			floor1_final_y.push(predicted as u32);
@@ -439,7 +436,7 @@ fn floor_one_curve_compute_amplitude(floor1_y :&[u32], fl :&FloorTypeOne) -> (Ve
 	return (floor1_final_y, floor1_step2_flag);
 }
 
-static FLOOR1_INVERSE_DB_TABLE :&'static[f32] = &[
+static FLOOR1_INVERSE_DB_TABLE :&[f32] = &[
 	1.0649863e-07, 1.1341951e-07, 1.2079015e-07, 1.2863978e-07,
 	1.3699951e-07, 1.4590251e-07, 1.5538408e-07, 1.6548181e-07,
 	1.7623575e-07, 1.8768855e-07, 1.9988561e-07, 2.1287530e-07,
@@ -764,6 +761,7 @@ fn residue_packet_decode(rdr :&mut BitpackCursor, cur_blocksize :u16,
 	}
 }
 
+#[inline]
 fn inverse_couple(m :f32, a :f32) -> (f32, f32) {
 	if m > 0. {
 		if a > 0. {
@@ -784,27 +782,22 @@ fn inverse_couple(m :f32, a :f32) -> (f32, f32) {
 // this function in unsafe code, no idea
 fn dual_mut_idx<T>(v :&mut [T], idx_a :usize, idx_b :usize)
 		-> (&mut T, &mut T) {
-	if idx_a < idx_b {
-		let sl = &mut v[idx_a..idx_b + 1];
-		let mut it = sl.iter_mut();
-		(it.next().unwrap(), it.last().unwrap())
-	} else if idx_a > idx_b {
-		let sl = &mut v[idx_b..idx_a + 1];
-		let mut it = sl.iter_mut();
-		(it.next().unwrap(), it.last().unwrap())
-	} else {
-		panic!("not allowed, indices must be different!");
-	}
+	assert_ne!(idx_a, idx_b, "not allowed, indices must be different!");
+
+	let range = if idx_a < idx_b { idx_a..idx_b+1 } else { idx_b..idx_a+1 };
+	let segment = &mut v[range];
+	let (first, rest) = segment.split_first_mut().unwrap();
+	let (last, _) = rest.split_last_mut().unwrap();
+	(first, last)
 }
 
 fn dct_iv_slow(buffer :&mut [f32]) {
-	let mut mcos = Vec::with_capacity(16384);
-	let x = buffer.to_owned().clone();
+	let x = buffer.to_vec();
 	let n = buffer.len();
 	let nmask = (n << 3) - 1;
-	for i in 0 .. 8 * n {
-		mcos.push(f32::cos(::std::f32::consts::PI / 4. * (i as f32) / (n as f32)));
-	}
+	let mcos = (0 .. 8 * n)
+		.map(|i| (std::f32::consts::FRAC_PI_4 * (i as f32) / (n as f32)).cos())
+		.collect::<Vec<_>>();
 	for i in 0 .. n {
 		let mut acc = 0.;
 		for j in 0 .. n {
@@ -813,13 +806,14 @@ fn dct_iv_slow(buffer :&mut [f32]) {
 		buffer[i] = acc;
 	}
 }
+
 #[allow(dead_code)]
 fn inverse_mdct_slow(buffer :&mut [f32]) {
 	let n = buffer.len();
 	let n4 = n >> 2;
 	let n2 = n >> 1;
 	let n3_4 = n - n4;
-	let mut temp = buffer[0 .. n2].to_owned().clone();
+	let mut temp = buffer[0 .. n2].to_vec();
 	dct_iv_slow(&mut temp); // returns -c'-d, a-b'
 	for i in 0 .. n4 {
 		buffer[i] = temp[i + n4]; // a-b'
@@ -857,16 +851,26 @@ pub struct PreviousWindowRight {
 }
 
 impl PreviousWindowRight {
-	// Initialisation for new streams
+	/// Initialisation for new streams
 	pub fn new() -> Self {
 		return PreviousWindowRight{ data : None };
 	}
+	/// If the state is still uninitialized
+	pub fn is_empty(&self) -> bool {
+		self.data.is_none()
+	}
 }
 
 /**
 Returns the per-channel sample count of a packet if it were decoded.
 
 This operation is very cheap and doesn't involve actual decoding of the packet.
+
+Note: for the first packet in a stream, or in other instances when
+the `PreviousWindowRight` is reset, the decoding functions will return
+0 samples for that packet, while this function returns a different number.
+Please use the `PreviousWindowRight::is_empty` function or other methods
+to check for this case.
 */
 pub fn get_decoded_sample_count(ident :&IdentHeader, setup :&SetupHeader, packet :&[u8])
 		-> Result<usize, AudioReadError> {
@@ -939,7 +943,7 @@ pub fn read_audio_packet_generic<S :Samples>(ident :&IdentHeader, setup :&SetupH
 		&setup.codebooks, &setup.floors));
 
 	// Now calculate the no_residue vector
-	let mut no_residue = SmallVec::<[bool; 256]>::new();
+	let mut no_residue = TinyVec::<[bool; 32]>::new();
 	for fl in &decoded_floor_infos {
 		no_residue.push(fl.is_unused());
 	}
@@ -957,7 +961,7 @@ pub fn read_audio_packet_generic<S :Samples>(ident :&IdentHeader, setup :&SetupH
 	// Helper variable
 	let resid_vec_len = (n / 2) as usize;
 	for (i, &residue_number) in mapping.mapping_submap_residues.iter().enumerate() {
-		let mut do_not_decode_flag = SmallVec::<[bool; 256]>::new();
+		let mut do_not_decode_flag = TinyVec::<[bool; 32]>::new();
 		for (j, &mapping_mux_j) in mapping.mapping_mux.iter().enumerate() {
 			if mapping_mux_j as usize == i {
 				do_not_decode_flag.push(no_residue[j]);
@@ -974,11 +978,10 @@ pub fn read_audio_packet_generic<S :Samples>(ident :&IdentHeader, setup :&SetupH
 		let mut ch = 0;
 		for (j, &mapping_mux_j) in mapping.mapping_mux.iter().enumerate() {
 			if mapping_mux_j as usize == i {
-				let mut v = Vec::with_capacity(resid_vec_len as usize);
-				let vec_at_ch = &vectors[resid_vec_len * ch .. resid_vec_len * (ch + 1)];
 				// TODO get rid of this copy somehow...
-				v.extend_from_slice(vec_at_ch);
-				residue_vectors[j] = v;
+				let vec_at_ch = &vectors[resid_vec_len * ch .. resid_vec_len * (ch + 1)];
+				residue_vectors[j].clear();
+				residue_vectors[j].extend_from_slice(vec_at_ch);
 				ch += 1;
 			}
 		}
@@ -1100,7 +1103,7 @@ pub fn read_audio_packet_generic<S :Samples>(ident :&IdentHeader, setup :&SetupH
 				start..end
 			};
 
-			let prev = prev_chan[0..plen].into_iter();
+			let prev = prev_chan[0..plen].iter();
 
 			let (lhs, rhs) = {
 				if win_slope.len() < plen {
