@@ -92,28 +92,21 @@ impl From<FromUtf8Error> for HeaderReadError {
 	}
 }
 
-impl error::Error for HeaderReadError {
-	fn description(&self) -> &str {
-		match self {
-			&HeaderReadError::EndOfPacket => "End of packet reached.",
-			&HeaderReadError::NotVorbisHeader => "The packet is not a vorbis header",
-			&HeaderReadError::UnsupportedVorbisVersion => "The vorbis version is not supported",
-			&HeaderReadError::HeaderBadFormat => "Invalid header",
-			&HeaderReadError::HeaderBadType(_) => "Invalid/unexpected header type",
-			&HeaderReadError::HeaderIsAudio => "Packet seems to be audio",
-			&HeaderReadError::Utf8DecodeError => "UTF-8 decoding error",
-			&HeaderReadError::BufferNotAddressable => "Requested to create buffer of non-addressable size",
-		}
-	}
-
-	fn cause(&self) -> Option<&error::Error> {
-		None
-	}
-}
+impl error::Error for HeaderReadError {}
 
 impl fmt::Display for HeaderReadError {
 	fn fmt(&self, fmt: &mut fmt::Formatter) -> Result<(), fmt::Error> {
-		write!(fmt, "{}", error::Error::description(self))
+		let description = match self {
+			HeaderReadError::EndOfPacket => "End of packet reached.",
+			HeaderReadError::NotVorbisHeader => "The packet is not a vorbis header",
+			HeaderReadError::UnsupportedVorbisVersion => "The vorbis version is not supported",
+			HeaderReadError::HeaderBadFormat => "Invalid header",
+			HeaderReadError::HeaderBadType(_) => "Invalid/unexpected header type",
+			HeaderReadError::HeaderIsAudio => "Packet seems to be audio",
+			HeaderReadError::Utf8DecodeError => "UTF-8 decoding error",
+			HeaderReadError::BufferNotAddressable => "Requested to create buffer of non-addressable size",
+		};
+		write!(fmt, "{}", description)
 	}
 }
 
@@ -561,7 +554,7 @@ impl <'a> BitpackCursor <'a> {
 	}
 }
 
-static MAX_BASES_WITHOUT_OVERFLOW : &'static[u32] = &[
+static MAX_BASES_WITHOUT_OVERFLOW : &[u32] = &[
 	0xffffffff, 0xffffffff, 0x0000ffff, 0x00000659,
 	0x000000ff, 0x00000054, 0x00000028, 0x00000017,
 	0x0000000f, 0x0000000b, 0x00000009, 0x00000007,
@@ -571,7 +564,7 @@ static MAX_BASES_WITHOUT_OVERFLOW : &'static[u32] = &[
 	0x00000002, 0x00000002, 0x00000002, 0x00000002,
 	0x00000002, 0x00000002, 0x00000002, 0x00000002];
 
-static MAX_BASE_MAX_BITS_WITHOUT_OVERFLOW : &'static[u8] = &[
+static MAX_BASE_MAX_BITS_WITHOUT_OVERFLOW : &[u8] = &[
 	0x1f, 0x1f, 0x0f, 0x0a,
 	0x07, 0x06, 0x05, 0x04,
 	0x03, 0x03, 0x03, 0x02,
@@ -596,7 +589,7 @@ fn exp_fast(base :u32, exponent: u8) -> u32 {
 			// Okay, now we have to find out
 			// whether this matters or not.
 			// Check whether selfmul would have been needed.
-			if i < 7 && (exponent >> i + 1) > 0 {
+			if i < 7 && (exponent >> (i + 1)) > 0 {
 				panic!("Overflow when squaring for exp_fast, \
 					precondition violated!");
 			}
@@ -607,7 +600,7 @@ fn exp_fast(base :u32, exponent: u8) -> u32 {
 }
 
 /// Returns, as defined in the vorbis spec:
-/// "the greatest integer value v for which v to the power of [codebook_dimensions] is less than or equal to [codebook_entries]"
+/// "the greatest integer for which to `[return_value]` the power of `[codebook_dimensions]` is less than or equal to `[codebook_entries]`"
 /// Essentially an "nth-root" algorithm.
 /// About the speed:
 /// Probably its super-optimized as it uses no floats,
@@ -666,8 +659,8 @@ fn test_lookup1_values() {
 	assert_eq!(lookup1_values(1, 1), 1);
 	assert_eq!(lookup1_values(0, 15), 0);
 	assert_eq!(lookup1_values(0, 0), 0);
-	assert_eq!(lookup1_values(1, 0), ::std::u32::MAX);
-	assert_eq!(lookup1_values(400, 0), ::std::u32::MAX);
+	assert_eq!(lookup1_values(1, 0), std::u32::MAX);
+	assert_eq!(lookup1_values(400, 0), std::u32::MAX);
 }
 
 /// Reads a codebook which is part of the setup header packet.
@@ -748,11 +741,11 @@ fn read_codebook(rdr :&mut BitpackCursor) -> Result<Codebook, HeaderReadError> {
 			codebook_multiplicands.push(try!(rdr.read_dyn_u32(codebook_value_bits)));
 		}
 		Some(CodebookVqLookup {
-			codebook_lookup_type : codebook_lookup_type,
-			codebook_minimum_value : codebook_minimum_value,
-			codebook_delta_value : codebook_delta_value,
-			codebook_sequence_p : codebook_sequence_p,
-			codebook_multiplicands : codebook_multiplicands,
+			codebook_lookup_type,
+			codebook_minimum_value,
+			codebook_delta_value,
+			codebook_sequence_p,
+			codebook_multiplicands,
 		})
 	};
 	let codebook_vq_lookup_vec = codebook_lookup.as_ref().map(|lup| {
